﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WebfleetConnect.QueueService;

namespace WebfleetConnect
{
  /// <summary>
  /// Shows message details of a queue message. (specialized for aux device data messages).
  /// </summary>
  public partial class AuxDataMessageDlg : Form
  {
    class AuxItem
    {
      private String name;
      private String value;

      public String Name { get { return name; } set { name = value; } }
      public String Value { get { return value; } set { this.value = value; } }
    }

    public AuxDataMessageDlg()
    {
      InitializeComponent();
    }

    public DialogResult ShowMessage(QueueServiceData msg)
    {
      msgTypeLabel.Text = "" + msg.msgType;
      msgTextLabel.Text = "" + msg.msgText;
      if (msg.odometerSpecified)
        odometerLabel.Text = "" + msg.odometer;
      if (msg.auxDeviceData != null && msg.auxDeviceData.dataIn != null && msg.auxDeviceData.dataIn.eventTimeSpecified)
      {
        timeLabel.Text = "" + msg.auxDeviceData.dataIn.eventTime;
      }
      if (msg.pos != null)
      {
        if (msg.pos.latitudeSpecified && msg.pos.longitudeSpecified)
        {
          latLabel.Text = "" + msg.pos.latitude;
          lonLabel.Text = "" + msg.pos.longitude;
        }
      }

      List<AuxItem> auxList = new List<AuxItem>();
      if (msg.auxDeviceData != null)
      {
        if (msg.auxDeviceData.dataIn != null)
        {
          auxList.Add(new AuxItem { Name = "DeviceID", Value = msg.auxDeviceData.dataIn.deviceId });
          auxList.Add(new AuxItem { Name = "AuxData", Value = BitConverter.ToString(msg.auxDeviceData.dataIn.data) });
        }
        if (msg.auxDeviceData.dataReference != null)
        {
          if (msg.auxDeviceData.dataReference.correlationIdSpecified)
            auxList.Add(new AuxItem { Name = "CorrelationID", Value = "" + msg.auxDeviceData.dataReference.correlationId });
          if (msg.auxDeviceData.dataReference.referenceIdSpecified)
            auxList.Add(new AuxItem { Name = "ReferenceID", Value = "" + msg.auxDeviceData.dataReference.referenceId });
        }
        if (msg.auxDeviceData.dataOutStatus != null)
        {
          auxList.Add(new AuxItem { Name = "DataOut Status", Value = "" + msg.auxDeviceData.dataOutStatus.status });
        }
        if (msg.auxDeviceData.quotaInfo != null)
        {
          auxList.Add(new AuxItem { Name = "Quota Used", Value = "" + msg.auxDeviceData.quotaInfo.used });
          auxList.Add(new AuxItem { Name = "Quota Max", Value = "" + msg.auxDeviceData.quotaInfo.max });
        }
      }

      auxDataGridView.DataSource = auxList;
      return ShowDialog();
    }

  }
}
