﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using InTheHand.Net;
using InTheHand.Net.Bluetooth;
using InTheHand.Net.Sockets;
using System.IO;
using WeifenLuo.WinFormsUI.Docking;

namespace WebfleetConnect
{
  public partial class BluetoothDialog : DockContent
  {
    Settings settings;
    BluetoothCntrlr bt;

    BindingSource clientDataSource = new BindingSource();
    List<ClientDisplayModel> clientList = new List<ClientDisplayModel>();


    public class ClientDisplayModel
    {
      String addr; String guid;
      public ClientDisplayModel(String addr, String guid) { this.addr = addr; this.guid = guid; }
      public String Addr { get { return this.addr; } }
      public String Server_UUID { get { return this.guid; } }
    }

    public BluetoothDialog(Settings settings)
    {
      InitializeComponent();
      this.settings = settings;
      clientDataSource.DataSource = clientList;
      this.clientDataGridView.DataSource = clientDataSource;

      this.bt = new BluetoothCntrlr();
      this.bt.ConnectedToServer += new EventHandler(bt_ConnectedToServer);
      this.bt.DisconnectedFromServer += new EventHandler(bt_DisconnectedFromServer);
      this.bt.DataReceived += new BluetoothCntrlr.DataReceivedEventHandler(bt_DataReceived);
      this.bt.ClientConnected += new BluetoothCntrlr.ClientConnectedEventHandler(bt_ClientConnected);
      this.bt.ClientDisConnected += new BluetoothCntrlr.ClientDisConnectedEventHandler(bt_ClientDisConnected);
      this.bt.ServerStarted += new BluetoothCntrlr.ServerStartedEventHandler(bt_ServerStarted);
      this.bt.ServerStopped += new BluetoothCntrlr.ServerStoppedEventHandler(bt_ServerStopped);

      this.bdAddrTextBox.Text = this.settings.lastBtAddr;

    }

    void bt_ServerStopped(object sender, BluetoothCntrlr.ServerStoppedEvent e)
    {
      this.Invoke((MethodInvoker)delegate { this.serverListBox.Items.Remove(e.ServiceGuid); });
    }

    void bt_ServerStarted(object sender, BluetoothCntrlr.ServerStartedEvent e)
    {
      this.Invoke((MethodInvoker)delegate { this.serverListBox.Items.Add(e.ServiceGuid); });
    }

    void bt_ClientDisConnected(object sender, BluetoothCntrlr.ClientDisConnectedEvent e)
    {
      this.Invoke((MethodInvoker)delegate 
        {
          foreach (ClientDisplayModel item in this.clientList)
          {
            if (item.Addr == e.RemoteAddress)
            {
              this.clientList.Remove(item);
              this.clientDataSource.ResetBindings(false);
              break;
            }
          }
        });
    }

    void bt_ClientConnected(object sender, BluetoothCntrlr.ClientConnectedEvent e)
    {
      this.Invoke((MethodInvoker)delegate 
        {
          this.clientList.Add(new ClientDisplayModel(e.RemoteAddress, e.LocalGuid.ToString()));
          this.clientDataSource.ResetBindings(false);
        });
    }

    private String MakeListboxDisplayString(String addr, Guid guid)
    {
      return addr + " (to " + guid + ")";
    }


    void bt_DataReceived(object sender, BluetoothCntrlr.DataReceivedEvent e)
    {
      System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
      String str = enc.GetString(e.Data);
      // Invoke in GUI thread
      ShowReceivedDataFx fx = new ShowReceivedDataFx(ShowReceivedData);
      this.Invoke(fx, new object[] { str });
    }

    void bt_DisconnectedFromServer(object sender, EventArgs e)
    {
        this.Invoke((MethodInvoker)delegate
        {
            this.connectButton.Enabled = true;
            this.disconnectButton.Enabled = false;
            this.clientStatusLabel.Text = "Disconnected";
            AdjustSendButtonState();
        });
    }

    void bt_ConnectedToServer(object sender, EventArgs e)
    {
        this.Invoke((MethodInvoker)delegate
        {
            this.connectButton.Enabled = false;
            this.disconnectButton.Enabled = true;
            this.clientStatusLabel.Text = "Connected";
            AdjustSendButtonState();
        });
    }

    private delegate void ShowReceivedDataFx(String data);
    private void ShowReceivedData(String data)
    {
        this.receivedDataTextBox.Text =
        this.receivedDataTextBox.Text + data;
    }


    private void connectButton_Click(object sender, EventArgs e)
    {
      ConnectAsClient(this.bdAddrTextBox.Text, (int)this.channelUpDown.Value, this.clientUuidTextBox.Text);
    }

    private void ConnectAsClient(String addr, int channel, string uuid)
    {
      try
      {
        this.bt.ConnectAsClient(addr, channel, uuid);
      }
      catch (Exception e)
      {
        MessageBox.Show(e.Message, "Error");
      }
    }

    private void disconnectButton_Click(object sender, EventArgs e)
    {
      this.bt.DisconnectFromServer();
    }

    private void sendButton_Click(object sender, EventArgs e)
    {
      System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
      byte[] data = enc.GetBytes(this.dataTextBox.Text);
      if (tabControl.SelectedIndex == 0)
      {
        // Client tab selected
        this.bt.SendData(data);
      }
      else
      {
        // Server tab selected
        if (this.clientDataGridView.CurrentRow  != null && this.clientDataGridView.CurrentRow.Index >= 0)
        {
          this.bt.SendData(data, this.clientList[this.clientDataGridView.CurrentRow.Index].Addr);
        }
      }
    }

    private void clearButton_Click(object sender, EventArgs e)
    {
      this.receivedDataTextBox.Text = "";
    }

    private void startServerButton_Click(object sender, EventArgs e)
    {
      try
      {
        this.bt.StartServer(this.uuidTextBox.Text, this.serviceNameTextBox.Text, (int)this.channelServerUpDown.Value);
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "Error");
      }
    }

    private void disconnectFromServerButton_Click(object sender, EventArgs e)
    {
      if (this.clientDataGridView.CurrentRow.Index >= 0 )
      {
        this.bt.DisconnectClient(this.clientList[this.clientDataGridView.CurrentRow.Index].Addr);
      }
    }

    private void stopServerButton_Click(object sender, EventArgs e)
    {
      if (this.serverListBox.SelectedItem != null)
      {
        this.bt.StopServer((Guid)this.serverListBox.SelectedItem);
      }
    }

    private void serverListBox_SelectedIndexChanged(object sender, EventArgs e)
    {
      this.stopServerButton.Enabled = (this.serverListBox.SelectedItem != null);
    }

    private void clientDataGridView_CurrentCellChanged(object sender, EventArgs e)
    {
      this.disconnectFromServerButton.Enabled = (this.clientDataGridView.CurrentRow != null);
      AdjustSendButtonState();
    }

    private void AdjustSendButtonState()
    {
      this.sendButton.Enabled =
        (this.tabControl.SelectedIndex == 0 && this.bt.IsConectedToServer)
        ||
        (this.tabControl.SelectedIndex == 1 && this.clientDataGridView.CurrentRow != null);
    }

    private void tabControl_SelectedIndexChanged(object sender, EventArgs e)
    {
      AdjustSendButtonState();
    }

    private void BluetoothDialog_FormClosing(object sender, FormClosingEventArgs e)
    {
      this.settings.lastBtAddr = this.bdAddrTextBox.Text;
    }

    private void BluetoothDialog_FormClosed(object sender, FormClosedEventArgs e)
    {

    }

    private void clientUuidTextBox_TextChanged(object sender, EventArgs e)
    {
      this.channelUpDown.Enabled = (clientUuidTextBox.Text.Length == 0);
    }

  }
}
