﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.ServiceModel.Configuration;
using System.Configuration;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;


namespace WebfleetConnect
{
  /// <summary>
  /// Input fields for WEBFLEET credentials. This is an embedded child form on the main window.
  /// </summary>
  public partial class CredentialDialog : Form
  {
    Credentials creds = new Credentials();
    Settings settings;


    public CredentialDialog(Settings authSettings)
    {
      InitializeComponent();
      this.settings = authSettings;

    }

    public void Init()
    {
      FillServers();
      RestoreState();
    }

    
    private void FillServers()
    {
      // Automagically find all client endpoints defined in app.config
      var clientSection = ConfigurationManager.GetSection("system.serviceModel/client") as ClientSection;
      var endpointCollection = clientSection.Endpoints;
      var endpointNames = (from ChannelEndpointElement item in endpointCollection select item.Name).ToArray();

      Dictionary<string, HostEntry> dict = new Dictionary<string, HostEntry>();
      foreach (string name in endpointNames)
      {
        string[] split = name.Split('-');
        HostEntry he;
        if (dict.ContainsKey(split[0]))
        {
          he = dict[split[0]];
        }
        else
        {
          dict[split[0]] = he = new HostEntry { Name = split[0] };
        }
        if (split[1].Equals("orders"))
        {
          he.OrdersEndpoint = name;
        }
        else if (split[1].Equals("address"))
        {
          he.AddressEndpoint = name;
        }
        else if (split[1].Equals("messages"))
        {
          he.MessagesEndpoint = name;
        }
        else if (split[1].Equals("objects"))
        {
          he.ObjectsEndpoint = name;
        }
        else if (split[1].Equals("auxdata"))
        {
          he.AuxEndpoint = name;
        }
      }
      var array = dict.Values.ToArray();

      this.endPointComboBox.DisplayMember = "Name";
      this.endPointComboBox.DataSource = array;

      // hide server choice box if there's only 1 endpoint configured
      if (this.endPointComboBox.Items.Count < 2)
      {
        this.endPointComboBox.Visible = false;
        this.endpointLabel.Visible = false;
        this.endPointComboBox.SelectedIndex = 0;
      }

    }

    public HostEntry SelectedEndpoint
    {
      get
      {
        return this.endPointComboBox.Items[this.endPointComboBox.SelectedIndex] as HostEntry;
      }
    }

    public Credentials Credentials
    {
      get
      {
        creds.account = accountTextBox.Text;
        creds.user = userTextBox.Text;
        creds.password = passwordTextBox.Text;
        creds.apiKey = apiKeyTextBox.Text.Length > 0 ? apiKeyTextBox.Text : null;
        return creds;
      }
    }


    private void RestoreState()
    {
      if (this.settings.wasLoaded)
      {
        this.userTextBox.Text = this.settings.user;
        this.accountTextBox.Text = this.settings.account;
        this.apiKeyTextBox.Text = this.settings.apiKey;
        this.endPointComboBox.Text = this.settings.server;
        if (this.endPointComboBox.SelectedIndex == -1)
        {
          // prevent invalid selections if corrupt setting are loaded
          this.endPointComboBox.SelectedIndex = 0;
        }
      }
    }

    private void okButton_Click(object sender, EventArgs e)
    {

    }

    private void CredentialDialog_Load(object sender, EventArgs e)
    {
      Init();
    }


    private void CredentialDialog_Shown(object sender, EventArgs e)
    {
      if (this.settings.wasLoaded)
        this.passwordTextBox.Focus();
    }

  }
}
