﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WebfleetConnect
{
  /// <summary>
  /// Embedded child dialog which allows to select an object (vehicle).
  /// The list of vehicles is populated with the reesult from WF.connect's showObjectReport.
  /// </summary>
  public partial class ObjectSelectorDlg : Form, WfcWindow
  {
    Credentials creds;
    HostEntry selectedEndpoint;
    String selectedObject;
    Settings settings;

    public ObjectSelectorDlg(Settings settings)
    {
      InitializeComponent();
      this.settings = settings;
    }

    public void SetCredentials(Credentials creds, HostEntry selectedEndpoint)
    {
      this.creds = creds;
      this.selectedEndpoint = selectedEndpoint;
      this.Enabled = true;
      FillObjectList();
    }

    public String SelectedObject { get { return this.selectedObject; } }

    private void FillObjectList()
    {
      this.objectComboBox.Items.Clear();

      var objectClient = new ObjectsService.objectsAndPeopleReportingClient(this.selectedEndpoint.ObjectsEndpoint);
      var aParm = new ObjectsService.AuthenticationParameters
      {
        accountName = creds.account,
        userName = creds.user,
        password = creds.password,
        apiKey = creds.apiKey
      };
      var gParm = new ObjectsService.GeneralParameters();
      var objectFilter = new ObjectsService.ObjectFilterParameter();
      var response = objectClient.showObjectReport(aParm, gParm, objectFilter);
      this.objectComboBox.Items.Clear();
      if (response.statusCode == 0)
      {
        foreach (ObjectsService.ObjectReport obj in response.results)
        {
          this.objectComboBox.Items.Add(obj.objectNo);
        }
      }
      else
      {
        MessageBox.Show(response.statusCode + " " + response.statusMessage, "Error");
      }

      if (this.settings.lastObjectNo != null)
      {
        this.objectComboBox.SelectedItem = this.settings.lastObjectNo;
      }
      else
      {
        this.objectComboBox.SelectedIndex = 0;
      }
    }

    private void objectComboBox_SelectedIndexChanged(object sender, EventArgs e)
    {
      this.selectedObject = this.objectComboBox.SelectedItem as String;
      this.settings.lastObjectNo = this.selectedObject;
      OnSelectedObjectChanged();
    }


    public event EventHandler SelectedObjectChanged;

    protected void OnSelectedObjectChanged()
    {
      if (SelectedObjectChanged != null)
      {
        SelectedObjectChanged(this, new EventArgs());
      }
    }


  }
}
